from time import sleep
from uuid import uuid4
import random
from datetime import datetime
import pytest

import tator


def random_localization(project, box_type, video_obj, post=False):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {
        "test_bool": random.choice([False, True]),
        "test_int": random.randint(-1000, 1000),
        "test_float": random.uniform(-1000.0, 1000.0),
        "test_enum": random.choice(["a", "b", "c"]),
        "test_string": str(uuid4()),
        "test_datetime": datetime.now().isoformat(),
        "test_geopos": [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
    }
    out = {
        "x": x,
        "y": y,
        "width": w,
        "height": h,
        "project": project,
        "type": box_type,
        "media_id": video_obj.id,
        "frame": random.randint(0, video_obj.num_frames - 1),
    }
    if post:
        out = {**out, **attributes}
    else:
        out["attributes"] = attributes
    return out


def add_attribute_helper(tator_api, type_getter, type_id, dtype):
    new_attr_name = f"New {dtype} {uuid4()}"
    entity_type = type_getter(type_id)

    # Make sure the new attribute does not exist already
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    addition = {
        "entity_type": f"{type(entity_type).__name__}",
        "addition": {"name": new_attr_name, "dtype": dtype},
    }
    if dtype == "enum":
        addition["addition"]["choices"] = ["a", "b", "c"]
    tator_api.add_attribute(id=type_id, attribute_type_spec=addition)
    entity_type = type_getter(type_id)

    # Check for added attribute
    assert any(attr.name == new_attr_name for attr in entity_type.attribute_types)


def add_invalid_attribute_helper(tator_api, type_getter, type_id):
    new_attr_name = f"New invalid attribute {uuid4()}"
    entity_type = type_getter(type_id)

    # Make sure the new attribute does not exist already
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    addition = {
        "entity_type": f"{type(entity_type).__name__}",
        "addition": {"name": new_attr_name, "dtype": "unknown"},
    }

    # Adding an attribute with an invalid `dtype` should raise an exception
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException) as excinfo:
        tator_api.add_attribute(id=type_id, attribute_type_spec=addition)

    # Check the exeption message for expected content
    assert (
        "ValidationError: \\\"'unknown' is not one of ['bool', 'int', 'float', 'enum', 'string', 'datetime', 'geopos']\\\""
        in str(excinfo.value)
    )


def test_box_type_add_invalid_attribute(host, token, project, attribute_box_type):
    tator_api = tator.get_api(host, token)
    add_invalid_attribute_helper(tator_api, tator_api.get_localization_type, attribute_box_type)


def test_state_type_add_invalid_attribute(host, token, project, state_type):
    tator_api = tator.get_api(host, token)
    add_invalid_attribute_helper(tator_api, tator_api.get_state_type, state_type)


def test_video_type_add_invalid_attribute(host, token, project, attribute_video_type):
    tator_api = tator.get_api(host, token)
    add_invalid_attribute_helper(tator_api, tator_api.get_media_type, attribute_video_type)


dtypes = ["int", "bool", "float", "string", "enum", "datetime", "geopos"]


@pytest.mark.parametrize("dtype", dtypes)
def test_box_type_add_attribute(host, token, project, attribute_box_type, dtype):
    tator_api = tator.get_api(host, token)
    add_attribute_helper(tator_api, tator_api.get_localization_type, attribute_box_type, dtype)


@pytest.mark.parametrize("dtype", dtypes)
def test_state_type_add_attribute(host, token, project, state_type, dtype):
    tator_api = tator.get_api(host, token)
    add_attribute_helper(tator_api, tator_api.get_state_type, state_type, dtype)


@pytest.mark.parametrize("dtype", dtypes)
def test_video_type_add_attribute(host, token, project, attribute_video_type, dtype):
    tator_api = tator.get_api(host, token)
    add_attribute_helper(tator_api, tator_api.get_media_type, attribute_video_type, dtype)


def test_add_enum_without_choices(host, token, project, attribute_video_type):
    dtype = "enum"
    tator_api = tator.get_api(host, token)
    new_attr_name = f"New {dtype} {uuid4()}"
    entity_type = tator_api.get_media_type(attribute_video_type)

    # Make sure the new attribute does not exist already
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    addition = {
        "entity_type": "MediaType",
        "addition": {"name": new_attr_name, "dtype": dtype},
    }

    # Adding an enum attribute without a `choices` field should raise an exception
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException) as excinfo:
        tator_api.add_attribute(id=attribute_video_type, attribute_type_spec=addition)

    # Check the exeption message for expected content
    assert "ValueError: enum attribute type definition missing 'choices' field" in str(
        excinfo.value
    )


def test_add_same_attribute_twice(host, token, project, line_type):
    tator_api = tator.get_api(host, token)
    new_attr_name = f"New attribute {uuid4()}"

    # Make sure the new attribute does not exist already
    entity_type = tator_api.get_localization_type(line_type)
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    addition = {
        "entity_type": "LocalizationType",
        "addition": {"name": new_attr_name, "dtype": "int"},
    }
    tator_api.add_attribute(id=line_type, attribute_type_spec=addition)
    entity_type = tator_api.get_localization_type(line_type)

    # Check for added attribute
    assert any(attr.name == new_attr_name for attr in entity_type.attribute_types)

    # Adding the same attribute a second time should raise an exception
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException) as excinfo:
        tator_api.add_attribute(id=line_type, attribute_type_spec=addition)

    # Check the exeption message for expected content
    assert "but one with that name already exists" in str(excinfo.value)


def test_add_same_attribute_with_different_dtypes(
    host, token, project, line_type, attribute_box_type
):
    tator_api = tator.get_api(host, token)
    new_attr_name = f"New attribute {uuid4()}"

    # Make sure the new attribute does not exist already
    entity_type = tator_api.get_localization_type(line_type)
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    entity_type = tator_api.get_localization_type(attribute_box_type)
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    addition = {
        "entity_type": "LocalizationType",
        "addition": {"name": new_attr_name, "dtype": "int"},
    }
    tator_api.add_attribute(id=line_type, attribute_type_spec=addition)
    entity_type = tator_api.get_localization_type(line_type)

    # Check for added attribute
    assert any(attr.name == new_attr_name for attr in entity_type.attribute_types)

    # Adding the same attribute with a different `dtype` should raise an exception
    addition["addition"]["dtype"] = "string"
    with pytest.raises(tator.openapi.tator_openapi.exceptions.ApiException) as excinfo:
        tator_api.add_attribute(id=attribute_box_type, attribute_type_spec=addition)

    # Check the exeption message for expected content
    assert (
        "but another entity type has already defined this attribute name with a different dtype"
        in str(excinfo.value)
    )


# @pytest.mark.skip(reason="Disabled")
@pytest.mark.parametrize("dtype", ["string", "bool"])
def test_box_type_attribute_addition_es(
    host, token, project, attribute_video, attribute_box_type, dtype
):
    tator_api = tator.get_api(host, token)
    video_obj = tator_api.get_media(attribute_video)

    num_localizations = 2
    boxes = [
        random_localization(project, attribute_box_type, video_obj, post=True)
        for _ in range(num_localizations)
    ]
    box_ids = [
        box_id
        for response in tator.util.chunked_create(
            tator_api.create_localization_list, project, localization_spec=boxes
        )
        for box_id in response.id
    ]

    assert len(box_ids) == len(boxes)

    # ES can be slow at indexing so wait for a bit.
    sleep(2)

    # Make sure the new attribute does not exist already
    if dtype == "string":
        value = str(uuid4()).lower()
        new_attr_name = f"New string {value}"
    elif dtype == "bool":
        value = False
        new_attr_name = f"New bool {uuid4()}"
    entity_type = tator_api.get_localization_type(attribute_box_type)
    assert all(attr.name != new_attr_name for attr in entity_type.attribute_types)
    addition = {
        "entity_type": "LocalizationType",
        "addition": {"name": new_attr_name, "dtype": dtype, "default": value},
    }
    tator_api.add_attribute(id=attribute_box_type, attribute_type_spec=addition)

    entity_type = tator_api.get_localization_type(attribute_box_type)

    # Check for added attribute
    assert any(attr.name == new_attr_name for attr in entity_type.attribute_types)

    # ES can be slow at indexing so wait for a bit.
    sleep(2)

    # Check for default value on existing instances
    params = {
        "type": attribute_box_type,
        "attribute": [f"{new_attr_name}::{str(value).lower()}"],
        "force_es": 1,
    }
    boxes = tator_api.get_localization_list(project, **params)

    assert len(box_ids) == len(boxes)

    for box in boxes:
        assert box.attributes[new_attr_name] == value

    # Clean up
    params = {"media_id": [attribute_video], "type": attribute_box_type}
    tator_api.delete_localization_list(project, **params)



import logging
import os

from ..openapi.tator_openapi.models import MessageResponse
from ..util.get_api import get_api
from ..util.get_parser import get_parser

logger = logging.getLogger(__name__)

def parse_args():
    parser = get_parser()
    parser.add_argument('--input', type=str, help="Path to file containing media ID.")
    return parser.parse_args()

def delete_media(host, token, media_id):
    """ Deletes a media object.

    :param host: Host URL.
    :param token: API token.
    :param media_id: Unique integer identifying a media.
    """
    api = get_api(host, token)
    response = api.delete_media(media_id)
    assert isinstance(response, MessageResponse)

if __name__ == '__main__':
    args = parse_args()
    if os.path.exists(args.input):
        with open(args.input, 'r') as f:
            media_id = int(f.read())
            delete_media(args.host, args.token, media_id)
    else:
        logger.info(f"No media deleted, file {args.input} could not be found!")



import os
import math
from uuid import uuid1

from urllib.parse import urljoin
from urllib.parse import urlsplit

from ._upload_file import _upload_file
from .md5sum import md5sum

def upload_temporary_file(api, project, path, lookup=None, hours=24,
                          name=None, chunk_size=100*1024*1024):
    """ Upload a file to the temporary file storage location.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        for progress, response in tator.util.upload_temporary_file(api, project_id, path):
            print(f"Upload progress: {progress}%")
        print(response.message)

    :param api: :class:`tator.TatorApi` object.
    :param project: Unique integer identifying a project.
    :param path: Path to the file.
    :param lookup: [Optional] md5hash of lookup parameters.
    :param hours: [Optional] Number of hourse file is kept alive. Default is 24.
    :param name: [Optional] Name of temporary file in database. Defaults to basename of path.
    :param chunk_size: [Optional] Chunk size in bytes. Default is 100MB.
    :returns: Generator that yields tuple containing progress (0-100) and a
        response. The response is `None` until the last yield, when the response
        is the response object from :meth:`tator.util.TatorApi.create_temporary_file`.
    """
    if name is None:
        name = os.path.basename(path)

    if lookup is None:
        lookup = name

    for progress, upload_info in _upload_file(api, project, path):
        yield (progress, None)
    url = api.get_download_info(project, {'keys': [upload_info.key]})[0].url

    response = api.create_temporary_file(project, temporary_file_spec={
        "url": url,
        "name": name,
        "lookup": lookup,
        "hours": 24,
    })
    yield (100, response)




# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class Job(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'gid': 'str',
        'id': 'str',
        'nodes': 'list[JobNode]',
        'project': 'int',
        'start_time': 'datetime',
        'status': 'str',
        'stop_time': 'datetime',
        'uid': 'str',
        'user': 'int'
    }

    attribute_map = {
        'gid': 'gid',
        'id': 'id',
        'nodes': 'nodes',
        'project': 'project',
        'start_time': 'start_time',
        'status': 'status',
        'stop_time': 'stop_time',
        'uid': 'uid',
        'user': 'user'
    }

    def __init__(self, gid=None, id=None, nodes=None, project=None, start_time=None, status=None, stop_time=None, uid=None, user=None, local_vars_configuration=None):  # noqa: E501
        """Job - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._gid = None
        self._id = None
        self._nodes = None
        self._project = None
        self._start_time = None
        self._status = None
        self._stop_time = None
        self._uid = None
        self._user = None
        self.discriminator = None

        if gid is not None:
            self.gid = gid
        if id is not None:
            self.id = id
        if nodes is not None:
            self.nodes = nodes
        if project is not None:
            self.project = project
        self.start_time = start_time
        if status is not None:
            self.status = status
        self.stop_time = stop_time
        if uid is not None:
            self.uid = uid
        if user is not None:
            self.user = user

    @property
    def gid(self):
        """
        Group ID of the job.

        :return: The gid of this Job. 
        :rtype: str
        """
        return self._gid

    @gid.setter
    def gid(self, gid):
        """
        Group ID of the job.

        :param gid: The gid of this Job.
        :type: str
        """

        self._gid = gid

    @property
    def id(self):
        """
        Unique identifier of the job generated by Argo.

        :return: The id of this Job. 
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique identifier of the job generated by Argo.

        :param id: The id of this Job.
        :type: str
        """

        self._id = id

    @property
    def nodes(self):
        """

        :return: The nodes of this Job. 
        :rtype: list[JobNode]
        """
        return self._nodes

    @nodes.setter
    def nodes(self, nodes):
        """

        :param nodes: The nodes of this Job.
        :type: list[JobNode]
        """

        self._nodes = nodes

    @property
    def project(self):
        """
        Unique integer identifying a project.

        :return: The project of this Job. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying a project.

        :param project: The project of this Job.
        :type: int
        """

        self._project = project

    @property
    def start_time(self):
        """
        Start time of this job.

        :return: The start_time of this Job. 
        :rtype: datetime
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """
        Start time of this job.

        :param start_time: The start_time of this Job.
        :type: datetime
        """

        self._start_time = start_time

    @property
    def status(self):
        """
        Status of this job.

        :return: The status of this Job. 
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """
        Status of this job.

        :param status: The status of this Job.
        :type: str
        """

        self._status = status

    @property
    def stop_time(self):
        """
        Stop time of this job.

        :return: The stop_time of this Job. 
        :rtype: datetime
        """
        return self._stop_time

    @stop_time.setter
    def stop_time(self, stop_time):
        """
        Stop time of this job.

        :param stop_time: The stop_time of this Job.
        :type: datetime
        """

        self._stop_time = stop_time

    @property
    def uid(self):
        """
        Unique ID of the job.

        :return: The uid of this Job. 
        :rtype: str
        """
        return self._uid

    @uid.setter
    def uid(self, uid):
        """
        Unique ID of the job.

        :param uid: The uid of this Job.
        :type: str
        """

        self._uid = uid

    @property
    def user(self):
        """
        Unique integer identifying user who submitted the job.

        :return: The user of this Job. 
        :rtype: int
        """
        return self._user

    @user.setter
    def user(self, user):
        """
        Unique integer identifying user who submitted the job.

        :param user: The user of this Job.
        :type: int
        """

        self._user = user

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Job):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Job):
            return True

        return self.to_dict() != other.to_dict()



"""
Python Bindings and associated utilities for tator
online platform
"""

# import submodule to top-level first
import tator.openapi as openapi

# Shortcuts
from .util.get_api import get_api
from .util.get_parser import get_parser
from .util.download_media import download_media

from .version import __version__ as __version__

# Generated code imports to top-level
import tator.openapi.tator_openapi.models as models
import tator.openapi.tator_openapi.exceptions as exceptions
from tator.openapi.tator_openapi import TatorApi as api


# Finally import utility functions
import tator.util as util



import tempfile
import os

import tator

def test_temporary_file(host, token, project):
    tator_api = tator.get_api(host, token)

    all_temps = tator_api.get_temporary_file_list(project)
    assert  all_temps is not None
    assert len(all_temps) == 0
    
    with tempfile.NamedTemporaryFile(mode='w',suffix=".txt") as temp:
        temp.write("foo")
        temp.flush()
        for progress, response in tator.util.upload_temporary_file(tator_api, project, temp.name):
            print(f"Temporary file upload progress: {progress}%")
        assert isinstance(response, tator.models.CreateResponse)
        print(response.message)
        all_temps = tator_api.get_temporary_file_list(project)
        assert len(all_temps) == 1

    with tempfile.TemporaryDirectory() as temp_dir:
        temp_fp = os.path.join(temp_dir, "foo.txt")
        temp_element = tator_api.get_temporary_file_list(project)[0]
        for progress in tator.util.download_temporary_file(tator_api, temp_element, temp_fp):
            print(f"Temporary file download progress: {progress}%")
        with open(temp_fp, 'r') as temp_file:
            contents = temp_file.read()
            assert contents == "foo"



import requests
import math
import os
import logging
import requests

import tator

logger = logging.getLogger(__name__)

def _upload_file(api, project, path, media_id=None, filename=None, chunk_size=1024*1024*10, file_size=None):
    """ Uploads a file.

    :param api: `tator.TatorApi` object.
    :param project: Unique integer identifying a project.
    :param path: Path to file on disk.
    :param media_id: [Optional] Media ID if this is an upload for existing media.
    :param filename: [Optional] Filename (only used if media ID is given).
    :param chunk_size: [Optional] Upload chunk size in bytes.
    """
    MAX_RETRIES = 10 # Maximum retries on a given chunk.

    # Get number of chunks.
    if file_size is None:
        file_size = os.stat(path).st_size
    num_chunks = math.ceil(file_size / chunk_size)
    if num_chunks > 10000:
        chunk_size = math.ceil(file_size / 9000)
        logger.warning(f"Number of chunks {num_chunks} exceeds maximum of 10,000. Increasing "
                        "chunk size to {chunk_size}.")
        num_chunks = math.ceil(file_size / chunk_size)

    if path.startswith('https://') or path.startswith('http://') and filename:
        filename = filename.split('?')[0]
    # Get upload info.
    upload_kwargs = {'num_parts': num_chunks}
    if media_id is not None:
        upload_kwargs['media_id'] = media_id
    if filename is not None:
        upload_kwargs['filename'] = filename
    upload_info = api.get_upload_info(project, **upload_kwargs)

    # Functor to wrap around file versus URL
    def get_data(path):
        if path.startswith('https://') or path.startswith('http://'):
            return requests.get(path, stream=True).raw
        else:
            return open(path, 'rb')
    if num_chunks > 1:
        # Upload parts.
        parts = []
        last_progress = 0
        yield (last_progress, None)
        with get_data(path) as f:
            for chunk_count, url in enumerate(upload_info.urls):
                file_part = f.read(chunk_size)
                for attempt in range(MAX_RETRIES):
                    try:
                        response = requests.put(url, data=file_part)
                        etag = response.headers['ETag']
                        parts.append({'ETag': etag, 'PartNumber': chunk_count + 1})
                        break
                    except Exception as e:
                        logger.warning(f"Upload of {path} chunk {chunk_count} failed ({e})! Attempt "
                                       f"{attempt + 1}/{MAX_RETRIES}")
                        if attempt == MAX_RETRIES - 1:
                            raise Exception(f"Upload of {path} failed!")
                this_progress = round((chunk_count / num_chunks) *100,1)
                if this_progress != last_progress:
                    yield (this_progress, None)
                    last_progress = this_progress

        # Complete the upload.
        response = api.complete_upload(project, upload_completion_spec={
            'key': upload_info.key,
            'upload_id': upload_info.upload_id,
            'parts': parts,
        })
        if not isinstance(response, tator.models.MessageResponse):
            raise Exception(f"Upload completion failed!")
    else:
        # Upload in single request.
        with get_data(path) as f:
            data = f.read()
            for attempt in range(MAX_RETRIES):
                response = requests.put(upload_info.urls[0], data=data)
                if response.status_code == 200:
                    break
                else:
                    logger.warning(f"Upload of {path} failed ({response.text}) size={len(data)}! Attempt "
                                   f"{attempt + 1}/{MAX_RETRIES}")
                    if attempt == MAX_RETRIES - 1:
                        raise Exception(f"Upload of {path} failed!")

    # Return the parts and upload info.
    yield (100, upload_info)




#!/usr/bin/env python

import argparse
import subprocess
import os
import json
import logging
import tempfile

from PIL import Image

from ..util import get_api
from ..util._upload_file import _upload_file

from .transcode import get_length_info
from ..openapi.tator_openapi.models import MessageResponse

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

def parse_args():
    parser = argparse.ArgumentParser(description='Makes thumbnails for a video.')
    parser.add_argument('--host', type=str, default='https://www.tatorapp.com', help='Host URL.')
    parser.add_argument('--token', type=str, help='REST API token.')
    parser.add_argument('--media', type=int, help='Unique integer identifying a media.')
    parser.add_argument('input', type=str, help='Path to input file.')
    parser.add_argument("-o", "--output", type=str, help='Path to output thumbnail.')
    parser.add_argument("-g", "--gif", type=str, help='Path to output thumbnail gif.')
    return parser.parse_args()

def get_metadata(path):
    cmd = [
        "ffprobe",
        "-v","error",
        "-show_entries", "stream",
        "-print_format", "json",
        "-select_streams", "v",
        "{}".format(path)
    ]
    output = subprocess.run(cmd, stdout=subprocess.PIPE, check=True).stdout

    logger.info("Got info = {}".format(output))
    video_info = json.loads(output)
    stream = video_info["streams"][0]

    fps, num_frames = get_length_info(stream)
    # Fill in object information based on probe
    codec = stream["codec_name"]
    width = stream["width"]
    height = stream["height"]

    return (codec, fps, num_frames, width, height)

def make_thumbnails(host, token, media_id, video_path, thumb_path, thumb_gif_path):
    """ Makes thumbnails and gets metadata for original file.
    """
    # Get metadata for original file.
    codec, fps, num_frames, width, height = get_metadata(video_path)

    # Create thumbnail.
    cmd = ["ffmpeg", "-y", "-seekable", "0", "-i", video_path, "-vf", "scale=256:-1", "-vframes", "1", thumb_path]
    subprocess.run(cmd, check=True)

    with tempfile.TemporaryDirectory() as dirname:
        pts_scale = (fps / 3) * (10 / num_frames)

        # Create gif thumbnail.
        cmd1 = ["ffmpeg", "-y"]
        if num_frames > 10000:
            cmd2 = ["-skip_frame", "nokey"]
        else:
            cmd2 = []
        cmd3 = ["-seekable", "0", "-i", video_path, "-vf", f"scale=256:-1:flags=lanczos,setpts={pts_scale}*PTS",
                "-r", "3", os.path.join(dirname, "%09d.jpg")]
        cmd = cmd1 + cmd2 + cmd3
        subprocess.run(cmd, check=True)
        cmd = [
            "ffmpeg", "-y", "-r", "3", "-i", os.path.join(dirname, '%09d.jpg'), "-vf",
            "split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse",
            "-r", "3",
            thumb_gif_path
        ]
        subprocess.run(cmd, check=True)

    # Upload thumbnail and thumbnail gif.
    api = get_api(host, token)
    media_obj = api.get_media(media_id)
    for progress, thumbnail_info in _upload_file(api, media_obj.project, thumb_path,
                                                 media_id=media_id,
                                                 filename=os.path.basename(thumb_path)):
        pass
    for progress, thumbnail_gif_info in _upload_file(api, media_obj.project, thumb_gif_path,
                                                     media_id=media_id,
                                                     filename=os.path.basename(thumb_gif_path)):
        pass

    # Open images to get output resolution.
    thumb_image = Image.open(thumb_path)
    thumb_gif_image = Image.open(thumb_gif_path)

    # Create image definitions for thumbnails.
    thumb_def = {'path': thumbnail_info.key,
                 'size': os.stat(thumb_path).st_size,
                 'resolution': [thumb_image.height, thumb_image.width],
                 'mime': f'image/{thumb_image.format.lower()}'}
    thumb_gif_def = {'path': thumbnail_gif_info.key,
                     'size': os.stat(thumb_gif_path).st_size,
                     'resolution': [thumb_gif_image.height, thumb_gif_image.width],
                     'mime': f'image/{thumb_gif_image.format.lower()}'}

    response = api.create_image_file(media_id, role='thumbnail', image_definition=thumb_def)
    assert isinstance(response, MessageResponse)
    response = api.create_image_file(media_id, role='thumbnail_gif', image_definition=thumb_gif_def)
    assert isinstance(response, MessageResponse)

    # Update the media object.
    response = api.update_media(media_id, media_update={
        'num_frames': num_frames,
        'fps': fps,
        'codec': codec,
        'width': width,
        'height': height,
    })
    assert isinstance(response, MessageResponse)
    logger.info(f'Thumbnail upload done! {response.message}')

if __name__ == '__main__':
    args = parse_args()
    make_thumbnails(args.host, args.token, args.media, args.input, args.output, args.gif)
